import { writeFile } from 'fs/promises';
import semver from 'semver';
import createDebugMessages from 'debug';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 *
 */
export async function syncDependencies(param) {
    const debug = createDebugMessages('@deep-foundation/npm-automation:npm-pull');
    debug({ param });
    const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
    const { default: deepJson } = await import(deepJsonFilePath, { assert: { type: 'json' } });
    debug({ deepJson });
    const { default: packageJson } = await import(packageJsonPath, { assert: { type: 'json' } });
    debug({ packageJson });
    if (!packageJson.dependencies) {
        packageJson.dependencies = {};
    }
    const missingDependenciesFromDeepJson = deepJson.dependencies.filter((dependency) => !!packageJson.dependencies[dependency.name]);
    debug({ missingDependenciesFromDeepJson });
    missingDependenciesFromDeepJson.forEach((dependency) => {
        packageJson.dependencies = { ...packageJson.dependencies, [dependency.name]: `~${dependency.version}` };
    });
    if (missingDependenciesFromDeepJson.length > 0) {
        await writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
        console.log(`${missingDependenciesFromDeepJson.map((dependency) => dependency.name).join(', ')} are added to package.json because they exist in deep.json`);
    }
    deepJson.dependencies.forEach((dependency) => {
        const dependencyVersionWithoutRange = semver.minVersion(dependency.version)?.version;
        debug({ dependencyVersionWithoutRange });
        if (!dependencyVersionWithoutRange) {
            return;
        }
        ;
        const isDeepJsonVersionGreater = semver.gt(dependencyVersionWithoutRange, packageJson.dependencies[dependency.name]);
        if (isDeepJsonVersionGreater) {
            packageJson.dependencies[dependency.name] = `~${dependencyVersionWithoutRange}`;
        }
        else {
            deepJson.dependencies = { ...deepJson.dependencies, [dependency.name]: `${dependencyVersionWithoutRange}` };
        }
    });
    Object.entries(packageJson.dependencies).forEach(([dependencyName, dependencyVersion]) => {
        const dependencyVersionWithoutRange = semver.minVersion(dependencyVersion)?.version;
        debug({ dependencyVersionWithoutRange });
        if (!dependencyVersionWithoutRange) {
            return;
        }
        ;
        const deepJsonDependency = deepJson.dependencies.find(dependency => dependency.name === dependency.name);
        if (!deepJsonDependency)
            return;
        const deepJsonDependencyVersionWithoutRange = semver.minVersion(deepJsonDependency.version)?.version;
        debug({ deepJsonDependencyVersionWithoutRange });
        if (!deepJsonDependencyVersionWithoutRange) {
            return;
        }
        ;
        const isPackageJsonVersionGreater = semver.gt(dependencyVersionWithoutRange, deepJson.dependencies.find(dependency => dependency.name === dependency.name).version);
        if (isPackageJsonVersionGreater) {
            deepJson.dependencies = { ...deepJson.dependencies, [dependencyName]: `${dependencyVersionWithoutRange}` };
        }
        else {
            packageJson.dependencies[dependencyName] = `~${deepJsonDependencyVersionWithoutRange}`;
        }
    });
}
